package memegene;

import org.nfunk.jep.*;

/**Esta clase es una extension de la clase GA, y corresponde a una particularización para
 * buscar máximos o mínimos de funciones matemáticas de una variable independiente
 *
 * @author Mauricio Alarcón M.
 * @author Felipe Urra A.
 */
public class GAFunction extends GA{

    /**Guarda la funcion a la cual los algoritmos buscarán maximizar o minimizar*/
    String funcion;
    /** Objeto JEP, el cual parsea funciones en texto hacia su equivalente numérico*/
    JEP evaluador  = new JEP();
    GPanel panel;
    /** arreglo que guarda los valores de cada iteración del algoritmo, para poder graficar     */
    double[] valores;

    /** Constructor
     *
     * @param individuos El numero de soluciones que competiran en los algoritmos
     * @param pmut Probabilidad de mutación
     * @param min booleano que indica true si se desea minimizar
     * @param a Rango inferior de la variable independiente de la funcion
     * @param b Rango superior de la variable independiente de la funcion
     * @param precision Cantidad de dígitos sucesores a la coma flotante de la variable
     *                  independiente de la funcion.
     * @param panel GPanel para graficar los resultados del algoritmo
     */
    GAFunction(int individuos,float pmut,boolean min,float a,float b, int precision,GPanel panel){
        super(individuos,pmut,min);
        this.setNumGenes(this.calculaBits(a, b, precision));
        this.panel = panel;
        setInterval(a,b);
        evaluador.addStandardFunctions();
        evaluador.addStandardConstants();
        evaluador.addComplex();
        evaluador.addFunction("sen",new org.nfunk.jep.function.Sine());
        evaluador.addVariable("x",0);
        evaluador.setImplicitMul(true);  //permite 2x vez de 2*x
    }

    /**Forma de parametrizar el rango de la variable independiente para numeros flotantes hacia un vector de bits
     *
     * @param a rango inferior de la variable independiente
     * @param b rango inferior de la variable independiente
     * @param precision numero de dígitos sucesores a la coma flotante de la variable independiente
     * @return nummero de bits a utilizar en el vector de bits
     */
    public int calculaBits(float a, float b, int precision){
        int bits;
        double factor = (b-a)*Math.pow(10,precision);
        for(bits=1; bits > -1; bits++)
            if(Math.pow(2,bits-1) < factor && factor <= (Math.pow(2,bits)-1))
                break;
            return bits;
    }

    //setea fitness de cada individuo de la poblacion y guarda en variable 'elite' el índice del
    //individuo con mayor capacidad

    /**
     * Metodo que obtiene la capacidad de los individuos, y guarda en la variable "eilite"
     * el indice del individuo con mayor capacidad.
     */
    public void fitness(){
        float mejor=0;
        elite = 0;
        int largo = capacidad.length;
        for(int i=0;i < largo;i++){
            poblacion.elementAt(i).setFit(a,b);
            if(min)                              //se ajusta capacidad en caso de minimización
              capacidad[i] = super.ajustar(funcion(poblacion.elementAt(i).getFit())); //cambiar por abstracta
            else
              capacidad[i] = funcion(poblacion.elementAt(i).getFit());          //cambiar por abstracta

            if(i==0) mejor = capacidad[i];    //elite
            else
              if(capacidad[i] > mejor){    
                mejor = capacidad[i];
                elite = i;
              }
        }

    }

    /**
     * Establece la función a maximizar o minimzar
     * @param funcion La función como un String
     */
    void setFuncion(String funcion){
        evaluador.parseExpression(funcion);
    }

    /**
     * Obtiene el valor de la función definida para la clase
     * @param in valor de la variable independiente de la funcion
     * @return valor de la variable dependiente
     */
    float funcion(float in){                       
        evaluador.addVariable("x",in);
        return (float)evaluador.getValue();
    }        

    /**
     * Ejecuta el algoritmo genético.
     * @param iteraciones Numero de iteraciones
     * @param idSeleccion Que tipo de seleccion se utilizara, Elite, torneo o ranking
     * @param probabilidadTorneo Si se ejecuto torneo, probabilidad que gane el.
     */
    public void ejecutar(int iteraciones,int idSeleccion, float probabilidadTorneo){

        valores = new double[iteraciones];
        System.out.println("EJECUTAR GA!!");
        for(int i=0; i < iteraciones;i++){
            this.fitness();

            switch(idSeleccion){
                case 0:
                       this.elite();
                       break;
                case 1:
                       this.ranking();
                       break;
                case 2:
                       this.tournament(probabilidadTorneo);

            }

            this.mutacion();            
            
            valores[i] = funcion(poblacion.elementAt(elite).getFit());
            
            
        }
        this.fitness();
        panel.setSerie1(valores);
    }

    /**
     * Imprime el resultado del algoritmo
     */
    public void print(){

        System.out.println("-----Algoritmo Genético----\n");
        System.out.println("Mejor Individuo: "+poblacion.elementAt(elite).getFit());
        if(min){
          System.out.println("Salida: " +desajustar(capacidad[elite]));
        }
        else
          System.out.println("Salida: " +capacidad[elite]+"\n");
    }


    
}