package memegene;

import java.awt.Dimension;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.chart.ChartPanel;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
/**
 * Contenedor del gráfico el cual muestra los resultados de los algoritmos.
 *
 * @author Mauricio Alarcón
 * @author Felipe Urra
 */

public class GPanel extends JPanel{

    /** Conjunto de datos a graficar.    */
    XYSeriesCollection dataset = new XYSeriesCollection();

    /** Arreglos con puntos.    */
    double x[],y[],x2[],y2[];
    /** Tamaño máximo de datos. Está relacionado con timeWindowSize   */
    int size;
    /** Nombres auto explicativos.    */
    String nombre,ejex,ejey;
    /** Gráfico como tal.    */
    ChartPanel panel;
    /** Dimensiones del panel.    */
    int ancho,alto;

    XYSeries serie1 = new XYSeries("Genético");
    XYSeries serie2 = new XYSeries("Meméticos");

    /** Constructor de la clase
     *
     * @param N       Cantidad de puntos. Es timeWindowSize
     * @param nombre  Título del gráfico
     * @param ejex    Nombre del eje X
     * @param ejey    Nombre del eje Y
     */

    /**
     * Constructor
     *
     * @param N Ventana de numeros a gráficar en el eje de las abscisas
     * @param nombre Nombre el gráfico
     * @param ejex Nombre del eje X
     * @param ejey Nombre del eje Y
     */
    GPanel(int N,String nombre, String ejex, String ejey){
        this.setSize(100,100);
        this.nombre = nombre;
        this.ejex = ejex;
        this.ejey = ejey;
        this.size = N;
        x = new double[size];
        y = new double[size];
        x2 = new double[size];
        y2 = new double[size];
        dataset.addSeries(serie1);
        dataset.addSeries(serie2);

        setValues1();            //comenzara vacio, java inicializa en 0 los arreglos
        setValues2();
        panel = this.getPanelGrafico();
        this.setLayout(new BoxLayout(this,BoxLayout.X_AXIS));
        this.add(panel);
    }
   /**
    * Elimina los registros del gráfico, de la serie 1
    */
    private void limpiarArreglos1(){

        for(int i = 0;i<x.length;i++){
            x[i]=0;
            y[i]=0;
        }
    }

    /**
     * Elimina los registros del gráfico, de la serie 1
     */
    private void limpiarArreglos2(){    

        for(int i = 0;i<x2.length;i++){
            x2[i]=0;
            y2[i]=0;
        }
    }
    /**
     * Setea los valores de la serie 1
     * @param arreglo de valores para cada indice del arreglo.
     */
    public void setSerie1(double valores[]){
        for(int i = 0;i<valores.length;i++){
            this.serie1.add(i,valores[i]);
        }
    }
    /**
     * Setea los valores de la serie 2
     * @param valores arreglo de valores para cada indice del arreglo.
     */
    public void setSerie2(double valores[]){
        for(int i = 0;i<valores.length;i++){
            this.serie2.add(i,valores[i]);
        }
    }

    /**
     * Setea valores de la serie 1
     */
    private void setValues1(){
        int i;
        serie1.clear();
        for(i=0;i<x.length;i++){
            this.serie1.add(this.y[i],this.x[i]); //agregamos los N valores
        }
    }

    /**
     * Setea valores de la serie 2
     */
    private void setValues2(){
        int i;
        serie2.clear();
        for(i=0;i<x2.length;i++){
            this.serie2.add(this.y2[i],this.x2[i]); //agregamos los N valores
        }
    }

/**
 *  Setea los arreglos de la serie 1.
 * @param x   Arreglo de tipo double para eje X.
 * @param y   Arreglo de tipo double para eje Y.
 *
 */
    void setArrays1(double x[],double y[]){
        this.x = x;
        this.y = y;
    }
/**
 *  Setea los arreglos de la serie 2.
 * @param x   Arreglo de tipo double para eje X.
 * @param y   Arreglo de tipo double para eje Y.
 *
 */
    void setArrays2(double x2[],double y2[]){
        this.x2 = x2;
        this.y2 = y2;
    }
/**
 * Método para ingresar valores a los arreglos de la serie 1.
 * @param x Valor a ingresar para arreglo de eje X.
 * @param y Valor a ingresar para arreglo de eje Y.
 */
    public void ingresarValor1(double x,double y){  //se ingresaran pares "x,y" "x,t" "y,t" uno a uno
        int i;
        for(i=0;i<this.x.length-1;i++){ //se corre todo a la izquierda
            this.x[i]=this.x[i+1];
            this.y[i]=this.y[i+1];
        }
        this.x[this.x.length-1]=x;        //se ingresa el ultimo valor
        this.y[this.y.length-1]=y;

        this.setValues1();               //se crea el dataset
    }
/**
 * Método para ingresar valores a los arreglos de la serie 2.
 * @param x Valor a ingresar para arreglo de eje X.
 * @param y Valor a ingresar para arreglo de eje Y.
 */
    public void ingresarValor2(double x2,double y2){  //se ingresaran pares "x,y" "x,t" "y,t" uno a uno
        int i;
        for(i=0;i<this.x2.length-1;i++){ //se corre todo a la izquierda
            this.x2[i]=this.x2[i+1];
            this.y2[i]=this.y2[i+1];
        }
        this.x2[this.x2.length-1]=x2;        //se ingresa el ultimo valor
        this.y2[this.y2.length-1]=y2;

        this.setValues2();               //se crea el dataset
    }
/**
 * Método para retornar el dataset.
 * @return Dataset retornado.
 */
    public XYSeriesCollection getDataSet(){
        return this.dataset;
    }


/**
 * Método que genera el gráfico y lo retorna para que se dibuje posteriormente.
 */
    JFreeChart getGrafico(){
        JFreeChart chart = ChartFactory.createXYLineChart(
                           this.nombre,this.ejey,
                           this.ejex, dataset, PlotOrientation.VERTICAL, true,
                           true, false);
        return chart;
    }
/**
 * Reactualiza la serie 1 del gráfico.
 * @param x Último valor de eje X.
 * @param y Último valor de eje Y.
 */
    void ActualizarGrafico1(double x, double y){
        ingresarValor1(x,y);
        this.panel = getPanelGrafico();
    }

    /**
 * Reactualiza la serie 2 del gráfico.
 * @param x Último valor de eje X.
 * @param y Último valor de eje Y.
 */
    void ActualizarGrafico2(double x2, double y2){
        ingresarValor2(x2,y2);
        this.panel = getPanelGrafico();
    }
/**
 * Retorna panel gráfico
 * @return panel gráfico
 */
    ChartPanel getPanelGrafico(){
        ChartPanel p = new ChartPanel(this.getGrafico(),false);
        //ChartPanel pp = new ChartPanel(this.getGrafico(),100,10,0,0,100,100,true,true,true,true,true,true);
        //p.setBounds(0, 0, 10, 10);
        //p.setMaximumDrawHeight(10);
        //p.setMaximumDrawWidth(10);
        p.setPreferredSize(new Dimension(ancho,alto));
        return p;
    }
/**
 * Setea panel gráfico de la serie 1.
 * @param x Última coordenada de eje X a ingersar antes de setear panel.
 * @param y Última coordenada de eje Y a ingersar antes de setear panel.
 */
    void setPanelGrafico1(double x, double y){
        ActualizarGrafico1(x,y);
        this.panel = getPanelGrafico();
    }
/**
 * Setea panel gráfico de la serie 2.
 * @param x Última coordenada de eje X a ingersar antes de setear panel.
 * @param y Última coordenada de eje Y a ingersar antes de setear panel.
 */
    void setPanelGrafico2(double x2, double y2){
        ActualizarGrafico2(x2,y2);
        this.panel = getPanelGrafico();
    }
    /**
     * Método que resetea la serie 1 del gráfico. Borra dataset y arreglos.
     */
    void resetearGrafico1(){
        this.serie1.clear();
        this.limpiarArreglos1();
        this.setValues1();
    }
/**
     * Método que resetea la serie 2 del gráfico. Borra dataset y arreglos.
     */
    void resetearGrafico2(){
        this.serie2.clear();
        this.limpiarArreglos2();
        this.setValues2();
    }
    /**
     * Setea panel gráfico recién reseteado. Llama a resetearGrafico()
     */
    void setPanelGraficoReseteado1(){

        this.resetearGrafico1();
        this.panel = getPanelGrafico();
    }

    void setPanelGraficoReseteado2(){

        this.resetearGrafico2();
        this.panel = getPanelGrafico();
    }

}
