package memegene;

import java.util.Random;

/** Clase que representa a un individuo.
 *
 * @author Mauricio Alarcón M.
 * @author Felipe Urra A.
 */
public class Individuo {
    /** Arreglo de números binarios que representa el genotipo del individuo. */
    public int[] genes;
    /** Fenotipo del individuo.*/
    private float fitness;

    /** Constructor.*/
    Individuo() {

    }
    
    /** Crea un individuo con genotipo aleatorio.
     *
     * @param num Cantidad de genes que contendrá el genotipo del individuo.
     */
    Individuo(int num) {
        Random r = new Random();
        this.genes = new int[num];
        
        for(int i=0; i < num; i++)
            genes[i] = r.nextInt(2);
        
    }
    /** Retorna valor de fenotipo.
     *
     * @return
     */
    public float getFit(){
      return fitness;
    }
    /** Setea el fenotipo del individuo. 
     *
     * @param a Rango mínimo de la escala de valores de la variable de la función a optimizar.
     * @param b Rango máximo de la escala de valores de la variable de la función a optimizar.
     */
    public void setFit(float a, float b){
        this.fitness = fit(a,b);
    }
    /** Codifica los genes como un valor en la escala de la variable de la función a optimizar.
     *
     * @param a Rango mínimo de la escala de valores de la variable de la función a optimizar.
     * @param b Rango máximo de la escala de valores de la variable de la función a optimizar.
     * @return fenotipo del individuo.
     */
    private float fit(float a, float b){
        double res;
        res = a + cromosomasToDecimal(this.genes)*((b-a)/(Math.pow(2,genes.length)-1));
        return (float)res;
    }
    /** Transforma un número binario a un valor decimal.
     * 
     * @param cromosomas Arreglo que representa un número binario.
     * @return Número decimal.
     */
    private int cromosomasToDecimal(int cromosomas[]){
        int numero = 0;
        int potencia = 0;
        
        for(int i=cromosomas.length-1;  i >=0; i--){
            numero += Math.pow(2,potencia)*cromosomas[i];
            potencia++;
        }
        
        return numero;
    }
    /** Clona un individuo.
     *
     * @return Individuo clonado.
     */
    public Individuo clone(){
        
        Individuo clon = new Individuo();
        
        clon.genes = (int []) this.genes.clone();
        
        clon.fitness = this.fitness;
        
        return clon;
    }
   
}
