package memegene;

import java.util.Random;
import java.util.Date;

/** Clase que maneja probabilidades y elecciones de tipo ruleta con individuos.
 *
 * @author Mauricio Alarcón M.
 * @author Felipe Urra A.
 */
public class Ruleta {
    /** Número de individuos.*/
    int cantidad;    //cantidad de individuos.
    /** Arreglo con fitness de individuos.*/
    float[] capacidad;
    /** Suma total de las capacidades.*/
    float total;
    /** Arreglo por porcentaje de aporte al total de fitness. Representa la función de densidad de probabilidad*/
    float[] porcentajeCapacidad;
    /** Arreglo con probabilidades que representa la función de densidad de probabilidad acumulada */
    float[] probabilidades;
    /** Objeto para números aleatorios.*/
    Random r;
    /** Constructor.
     * 
     * @param cap   Arreglo de capacidades de individuos.
     * @param elite Flag que indica si el método de selección es elite.
     */
    Ruleta(float[] cap, boolean elite) {
        cantidad = cap.length;
        capacidad = cap.clone();
        porcentajeCapacidad = new float[cantidad];
        
        total = 0;
        
        for(int i=0;i < cantidad;i++)   //se calcula total
            total += capacidad[i];     //total
        
        for(int i=0; i < cantidad; i++)
            porcentajeCapacidad[i] = (float)(capacidad[i]/total);  //se asigna porcentaje de participacion
        
        probabilidades = new float[cantidad];
        for(int i=0; i < cantidad-1; i++){
            if(i == 0) probabilidades[i] = porcentajeCapacidad[i];
            else probabilidades[i] = probabilidades[i-1] + porcentajeCapacidad[i];
        }
        probabilidades[cantidad-1] = 1;
        
        r = new Random();
        r.setSeed(new Date().getTime());
    }
    /** Simula una ruleta con lanzamientos para obtener individuos candidatos.
     * 
     * @return Arreglo con ganadores de los lanzamientos.
     */
    public int[] go(){
        
        float valor;
        int [] ganadores = new int[cantidad];
        int j,k = 0;
        
        for(int i = 0; i < cantidad ; i++){
            
            valor = r.nextFloat();     //número sorteado

            for(j=0; j < cantidad;j++)
                if(probabilidades[j] > valor) {
                    break;    //se guarda indice de sorteado
                }
            ganadores[k] = j; k++; 
        }
        
        return ganadores;
        
    }
    /** Método que simula la ocurrencia de un evento que tiene cierta probabilidad.
     * 
     * @param probab Probabilidad de ocurrencia del evento.
     * @return True si ocurre y false si no ocurre.
     */
    boolean go(float probab){
        
        float p = r.nextFloat();
        
        if(p <= probab)
            return true;
        else
            return false;
    }
    /** Retorna un número aleatorio entre 0 y el argumento especificado
     *
     * @param num Argumento.
     * @return número aleatorio.
     */
    int lugar(int num){
        return r.nextInt(num);
    }
  
}
